#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from fbpcp.entity.mpc_instance import MPCInstance
from fbpcp.repository.instance_s3 import S3InstanceRepository
from fbpcp.repository.mpc_instance import MPCInstanceRepository
from fbpcp.service.storage_s3 import S3StorageService


class S3MPCInstanceRepository(MPCInstanceRepository):
    def __init__(self, storage_svc: S3StorageService, base_dir: str) -> None:
        self.repo = S3InstanceRepository(storage_svc, base_dir)

    def create(self, instance: MPCInstance) -> None:
        self.repo.create(instance)

    def read(self, instance_id: str) -> MPCInstance:
        return MPCInstance.loads_schema(self.repo.read(instance_id))

    def update(self, instance: MPCInstance) -> None:
        self.repo.update(instance)

    def delete(self, instance_id: str) -> None:
        self.repo.delete(instance_id)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from fbpcp.entity.mpc_instance import MPCInstance
from fbpcp.repository.instance_local import LocalInstanceRepository
from fbpcp.repository.mpc_instance import MPCInstanceRepository


class LocalMPCInstanceRepository(MPCInstanceRepository):
    def __init__(self, base_dir: str) -> None:
        self.repo = LocalInstanceRepository(base_dir)

    def create(self, instance: MPCInstance) -> None:
        self.repo.create(instance)

    def read(self, instance_id: str) -> MPCInstance:
        return MPCInstance.loads_schema(self.repo.read(instance_id))

    def update(self, instance: MPCInstance) -> None:
        self.repo.update(instance)

    def delete(self, instance_id: str) -> None:
        self.repo.delete(instance_id)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from fbpcp.entity.instance_base import InstanceBase
from fbpcp.service.storage_s3 import S3StorageService


class S3InstanceRepository:
    def __init__(self, s3_storage_svc: S3StorageService, base_dir: str) -> None:
        self.s3_storage_svc = s3_storage_svc
        self.base_dir = base_dir

    def create(self, instance: InstanceBase) -> None:
        if self._exist(instance.get_instance_id()):
            raise RuntimeError(f"{instance.get_instance_id()} already exists")

        filename = f"{self.base_dir}{instance.get_instance_id()}"
        # Use pickle protocol 0 to make ASCII only bytes that can be safely decoded into a string
        self.s3_storage_svc.write(filename, instance.dumps_schema())

    def read(self, instance_id: str) -> str:
        if not self._exist(instance_id):
            raise RuntimeError(f"{instance_id} does not exist")

        filename = f"{self.base_dir}{instance_id}"
        return self.s3_storage_svc.read(filename)

    def update(self, instance: InstanceBase) -> None:
        if not self._exist(instance.get_instance_id()):
            raise RuntimeError(f"{instance.get_instance_id()} does not exist")

        filename = f"{self.base_dir}{instance.get_instance_id()}"
        # Use pickle protocol 0 to make ASCII only bytes that can be safely decoded into a string
        self.s3_storage_svc.write(filename, instance.dumps_schema())

    def delete(self, instance_id: str) -> None:
        if not self._exist(instance_id):
            raise RuntimeError(f"{instance_id} does not exist")

        filename = f"{self.base_dir}{instance_id}"
        self.s3_storage_svc.delete(filename)

    def _exist(self, instance_id: str) -> bool:
        return self.s3_storage_svc.file_exists(f"{self.base_dir}{instance_id}")



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from decimal import Decimal
from functools import reduce
from typing import Any, Dict, List

from fbpcp.entity.cloud_cost import CloudCost, CloudCostItem
from fbpcp.entity.cluster_instance import Cluster, ClusterStatus
from fbpcp.entity.container_instance import ContainerInstance, ContainerInstanceStatus
from fbpcp.entity.subnet import Subnet
from fbpcp.entity.vpc_instance import Vpc, VpcState


def map_ecstask_to_containerinstance(task: Dict[str, Any]) -> ContainerInstance:
    container = task["containers"][0]
    ip_v4 = (
        container["networkInterfaces"][0]["privateIpv4Address"]
        if len(container["networkInterfaces"]) > 0
        else None
    )

    status = container["lastStatus"]
    if status == "RUNNING":
        status = ContainerInstanceStatus.STARTED
    elif status == "STOPPED":
        if container.get("exitCode") == 0:
            status = ContainerInstanceStatus.COMPLETED
        else:
            status = ContainerInstanceStatus.FAILED
    else:
        status = ContainerInstanceStatus.UNKNOWN

    return ContainerInstance(task["taskArn"], ip_v4, status)


def map_esccluster_to_clusterinstance(cluster: Dict[str, Any]) -> Cluster:
    status = cluster["status"]
    if status == "ACTIVE":
        status = ClusterStatus.ACTIVE
    elif status == "INACTIVE":
        status = ClusterStatus.INACTIVE
    else:
        status = ClusterStatus.UNKNOWN

    tags = _convert_aws_tags_to_dict(cluster["tags"], "key", "value")
    return Cluster(cluster["clusterArn"], cluster["clusterName"], status, tags)


def map_ec2vpc_to_vpcinstance(vpc: Dict[str, Any]) -> Vpc:
    state = vpc["State"]
    if state == "pending":
        state = VpcState.PENDING
    elif state == "available":
        state = VpcState.AVAILABLE
    else:
        state = VpcState.UNKNOWN

    vpc_id = vpc["VpcId"]
    # some vpc instances don't have any tags
    tags = (
        _convert_aws_tags_to_dict(vpc["Tags"], "Key", "Value") if "Tags" in vpc else {}
    )

    return Vpc(vpc_id, state, tags)


def map_ec2subnet_to_subnet(subnet: Dict[str, Any]) -> Subnet:
    availability_zone = subnet["AvailabilityZone"]
    subnet_id = subnet["SubnetId"]
    tags = (
        _convert_aws_tags_to_dict(subnet["Tags"], "Key", "Value")
        if "Tags" in subnet
        else {}
    )
    return Subnet(subnet_id, availability_zone, tags)


def _convert_aws_tags_to_dict(
    tag_list: List[Dict[str, str]], tag_key: str, tag_value: str
) -> Dict[str, str]:
    return reduce(lambda x, y: {**x, **{y[tag_key]: y[tag_value]}}, tag_list, {})


def map_cecost_to_cloud_cost(cost_by_date: List[Dict[str, Any]]) -> CloudCost:
    total_cost_amount = Decimal(0)
    cost_items = {}
    for daily_result in cost_by_date:
        for group_result in daily_result.get("Groups"):
            amount = Decimal(group_result["Metrics"]["UnblendedCost"]["Amount"])
            total_cost_amount += amount
            cost_items[tuple(group_result["Keys"])] = (
                cost_items.get(tuple(group_result["Keys"]), 0) + amount
            )

    return CloudCost(
        total_cost_amount=total_cost_amount,
        details=[
            CloudCostItem(region=region, service=service, cost_amount=amount)
            for (region, service), amount in cost_items.items()
        ],
    )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from enum import Enum


class RegionName(Enum):
    AWS_US_EAST_1 = "us-east-1"
    AWS_US_EAST_2 = "us-east-2"
    AWS_US_WEST_1 = "us-west-1"
    AWS_US_WEST_2 = "us-west-2"
    # TODO will add more region name in later diff



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import abc
from typing import Type, TypeVar

from dataclasses_json import DataClassJsonMixin

T = TypeVar("T", bound="InstanceBase")


class InstanceBase(DataClassJsonMixin):
    @abc.abstractmethod
    def get_instance_id(self) -> str:
        pass

    def __str__(self) -> str:
        return self.to_json()

    def dumps_schema(self) -> str:
        return self.schema().dumps(self)

    @classmethod
    def loads_schema(cls: Type[T], json_schema_str: str) -> T:
        return cls.schema().loads(json_schema_str, many=None)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from pathlib import Path

from fbpcp.entity.instance_base import InstanceBase


class LocalInstanceRepository:
    def __init__(self, base_dir: str) -> None:
        self.base_dir = Path(base_dir)

    def create(self, instance: InstanceBase) -> None:
        if self._exist(instance.get_instance_id()):
            raise RuntimeError(f"{instance.get_instance_id()} already exists")

        path = self.base_dir.joinpath(instance.get_instance_id())
        with open(path, "w") as f:
            f.write(instance.dumps_schema())

    def read(self, instance_id: str) -> str:
        if not self._exist(instance_id):
            raise RuntimeError(f"{instance_id} does not exist")

        path = self.base_dir.joinpath(instance_id)
        with open(path, "r") as f:
            return f.read().strip()

    def update(self, instance: InstanceBase) -> None:
        if not self._exist(instance.get_instance_id()):
            raise RuntimeError(f"{instance.get_instance_id()} does not exist")

        path = self.base_dir.joinpath(instance.get_instance_id())
        with open(path, "w") as f:
            f.write(instance.dumps_schema())

    def delete(self, instance_id: str) -> None:
        if not self._exist(instance_id):
            raise RuntimeError(f"{instance_id} does not exist")

        self.base_dir.joinpath(instance_id).unlink()

    def _exist(self, instance_id: str) -> bool:
        return self.base_dir.joinpath(instance_id).exists()



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from enum import Enum


class ServiceName(Enum):
    AWS_MACIE = "Amazon Macie"
    AWS_S3 = "Amazon Simple Storage Service"
    # TODO will add more service name



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass
from enum import Enum
from typing import Any, Dict, List, Optional

from fbpcp.entity.container_instance import ContainerInstance
from fbpcp.entity.instance_base import InstanceBase


# TODO: T96692057 will delete MPCRole, keeping for now so that code doesn't break
# while fbpmp codebase expects it to exist
class MPCRole(Enum):
    SERVER = "SERVER"
    CLIENT = "CLIENT"


class MPCParty(Enum):
    SERVER = "SERVER"
    CLIENT = "CLIENT"


class MPCInstanceStatus(Enum):
    UNKNOWN = "UNKNOWN"
    CREATED = "CREATED"
    STARTED = "STARTED"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"
    CANCELED = "CANCELED"


@dataclass
class MPCInstance(InstanceBase):
    instance_id: str
    game_name: str
    mpc_party: MPCParty
    num_workers: int
    server_ips: Optional[List[str]]
    containers: List[ContainerInstance]
    status: MPCInstanceStatus
    game_args: Optional[List[Dict[str, Any]]]

    @classmethod
    def create_instance(
        cls,
        instance_id: str,
        game_name: str,
        num_workers: int,
        mpc_role: Optional[MPCRole] = None,
        mpc_party: Optional[MPCParty] = None,
        server_ips: Optional[List[str]] = None,
        containers: Optional[List[ContainerInstance]] = None,
        status: MPCInstanceStatus = MPCInstanceStatus.UNKNOWN,
        game_args: Optional[List[Dict[str, Any]]] = None,
    ) -> "MPCInstance":
        # TODO: T96692057 will delete mpc_role and make mpc_party required
        if mpc_party:
            party = mpc_party
        elif mpc_role:
            party = MPCParty.SERVER if mpc_role is MPCRole.SERVER else MPCParty.CLIENT
        else:
            raise ValueError("mpc_role or mpc_party should be specified")

        return cls(
            instance_id,
            game_name,
            party,
            num_workers,
            server_ips,
            containers or [],
            status,
            game_args,
        )

    def get_instance_id(self) -> str:
        return self.instance_id
